#ifndef LATTICEMOVESET_H_
#define LATTICEMOVESET_H_


#include "LatticeModel.hh"
#include "LatticeProtein.hh"

namespace ell
{

		/*! This class provides an interface for move set implementation
		 * to define the neighborhood in the energy landscape of LatticeProtein
		 * objects.
		 * 
		 * An object of this class is able to generate a neighbored LatticeProtein
		 * by applying on of its moves.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class LatticeMoveSet
	{
	protected:
			/*! the lattice model this move set bases on */
		const LatticeModel* lattice;
			
	public:
			/*! Constructs a LatticeMoveSet on a given lattice model.
			 * @param lattice has to be != NULL
			 */
		LatticeMoveSet(const LatticeModel* _lattice);
		
		virtual ~LatticeMoveSet();
		
			/*! Applies a special move to the given LatticeProtein on the 
			 * specified position and returns a new LatticeProtein object 
			 * as result.
			 * 
			 * @param todo the initial state
			 * @param position  the structure position where to apply the move
			 * @param moveIndex the move index (has to be less than the return 
			 * 					value of getMoveNumber())
			 * @return a new LatticeProtein object neighbored to the todo object
			 *
			 *  In case of invalid moves, the behaviour
			 *  of the program is not specified.
			 */
		virtual LatticeProtein* applyMove(const LatticeProtein* const todo, 
								  size_t position, size_t moveIndex) = 0;
									
			/*! Applies a special move to the given LatticeProtein on the 
			 * specified position INPLACE and returns the modified object
			 * as result.
			 * 
			 * @param todo the initial state to modify
			 * @param position  the structure position where to apply the move
			 * @param moveIndex the move index (has to be less than the return 
			 * 					value of getMoveNumber())
			 * @return the modified LatticeProtein 
			 *
			 *  In case of invalid moves, the behaviour
			 *  of the program is not specified.
			 */
		virtual LatticeProtein* applyMoveInPlace(LatticeProtein* todo, size_t position,
									size_t moveIndex) = 0;
		
			/*! Returns the number of available moves by this MoveSet. */
		virtual unsigned int getMoveNumber() const = 0;
	};

} // namespace ell

#include "LatticeMoveSet.icc"

#endif /*LATTICEMOVESET_H_*/
